import os

from flask import url_for
from invenio_records.api import Record
from oarepo_communities.record import CommunityRecordMixin
from oarepo_records_draft.record import InvalidRecordAllowedMixin, DraftRecordMixin
from oarepo_references.mixins import ReferenceEnabledRecordMixin
from oarepo_validate import SchemaKeepingRecordMixin, MarshmallowValidatedRecordMixin

from nr_nresults.constants import NRESULTS_ALLOWED_SCHEMAS, NRESULTS_PREFERRED_SCHEMA
from nr_nresults.marshmallow import NResultsMetadataSchemaV1

published_index_name = 'nr_nresults-nr-nresults-v1.0.0'
draft_index_name = 'draft-nr_nresults-nr-nresults-v1.0.0'
all_index_name = 'nr-all'

prefixed_published_index_name = os.environ.get('INVENIO_SEARCH_INDEX_PREFIX',
                                               '') + published_index_name
prefixed_draft_index_name = os.environ.get('INVENIO_SEARCH_INDEX_PREFIX', '') + draft_index_name
prefixed_all_index_name = os.environ.get('INVENIO_SEARCH_INDEX_PREFIX', '') + all_index_name


class NResultBaseRecord(SchemaKeepingRecordMixin,
                        MarshmallowValidatedRecordMixin,
                        ReferenceEnabledRecordMixin,
                        CommunityRecordMixin,
                        Record,
                        ):
    ALLOWED_SCHEMAS = NRESULTS_ALLOWED_SCHEMAS
    PREFERRED_SCHEMA = NRESULTS_PREFERRED_SCHEMA
    MARSHMALLOW_SCHEMA = NResultsMetadataSchemaV1


class PublishedNResultRecord(InvalidRecordAllowedMixin, NResultBaseRecord):
    index_name = published_index_name

    @property
    def canonical_url(self):
        return url_for('invenio_records_rest.nresults_item',
                       pid_value=self['control_number'], _external=True)


class DraftNResultRecord(DraftRecordMixin, NResultBaseRecord):
    index_name = draft_index_name

    @property
    def canonical_url(self):
        return url_for('invenio_records_rest.draft-nresults_item',
                       pid_value=self['control_number'], _external=True)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2019 CIS UCT Prague.
#
# CIS theses repository is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Default configuration."""

from __future__ import absolute_import, print_function

from invenio_records_rest.utils import allow_all

from nr_nresults.record import draft_index_name, published_index_name
from nr_nresults.search import NresultsRecordsSearch

RECORDS_DRAFT_ENDPOINTS = {
    'nresults': {
        'draft': 'draft-nresults',

        'pid_type': 'nrnrs',
        'pid_minter': 'nr_nresults',
        'pid_fetcher': 'nr_nresults',
        'default_endpoint_prefix': True,
        'max_result_window': 500000,
        'search_index': published_index_name,  # TODO: nestáhl se sám, podívat se na to
        'record_class': 'nr_nresults.record:PublishedNResultRecord',
        'list_route': '/nresults/',
        'publish_permission_factory_imp': allow_all,  # TODO: change this !!!
        'unpublish_permission_factory_imp': allow_all,
        'edit_permission_factory_imp': allow_all,
        'default_media_type': 'application/json',
        'links_factory_imp': 'oarepo_fsm.links:record_fsm_links_factory',
        'search_class': NresultsRecordsSearch
        # 'indexer_class': CommitingRecordIndexer,

    },
    'draft-nresults': {
        'pid_type': 'dnrnrs',
        'record_class': 'nr_nresults.record.DraftNResultRecord',
        'list_route': '/draft/nresults/',
        'search_index': draft_index_name,
        'links_factory_imp': 'oarepo_fsm.links:record_fsm_links_factory',
        'search_class': NresultsRecordsSearch
    }
}

FILTERS = {
}

POST_FILTERS = {
}

RECORDS_REST_FACETS = {
}

RECORDS_REST_SORT_OPTIONS = {
}

RECORDS_REST_DEFAULT_SORT = {
}

"""Set default sorting options."""



# -*- coding: utf-8 -*-
#
# This file is part of Invenio.
# Copyright (C) 2015-2018 CERN.
#
# Invenio is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.

"""Persistent identifier minters."""

from __future__ import absolute_import, print_function

from nr_common.minters import nr_id_minter

from nr_nresults.providers import NRNresultsIdProvider


def nr_nresults_id_minter(record_uuid, data):
    return nr_id_minter(record_uuid, data, nr_id_provider=NRNresultsIdProvider)



from invenio_search import RecordsSearch


class NresultsRecordsSearch(RecordsSearch):

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self._source = [
            'control_number', 'oarepo:validity.valid', 'oarepo:draft', 'title', 'dateIssued',
            'creator', 'resourceType', 'contributor', 'keywords', 'subject', 'abstract', 'state']
        self._highlight['title.cs'] = {}
        self._highlight['title._'] = {}
        self._highlight['title.en'] = {}